/*
 * Device support for Tektronix TDS3000-series digital oscilloscopes
 *
 ************************************************************************
 * Copyright (c) 2003 The University of Chicago, as Operator of Argonne
 * National Laboratory, and the Regents of the University of
 * California, as Operator of Los Alamos National Laboratory.
 * TDS3000 support is distributed subject to a Software License Agreement
 * found in file LICENSE that is included with this distribution.
 ************************************************************************
 *
 * Original Author: Nick Di Monte <npd@aps.anl.gov>
 */
 
/* devTDS3000.c,v 1.7 2005/11/22 22:48:06 npd Exp */

#include    <stdlib.h>
#include    <stdio.h>
#include    <string.h>
#include    <alarm.h>
#include    <recGbl.h>
#include    <devCommonGpib.h>
#include    <menuFtype.h>

#define DSET_AI         devAiTDS3000Gpib
#define DSET_AO         devAoTDS3000Gpib
#define DSET_LI         devLiTDS3000Gpib
#define DSET_LO         devLoTDS3000Gpib
#define DSET_BI         devBiTDS3000Gpib
#define DSET_BO         devBoTDS3000Gpib
#define DSET_MBBO       devMbboTDS3000Gpib
#define DSET_MBBI       devMbbiTDS3000Gpib
#define DSET_SI         devSiTDS3000Gpib
#define DSET_SO         devSoTDS3000Gpib
#define DSET_WF         devWfTDS3000Gpib

#include    <devGpib.h>

int TDS3000Debug = 0;          /* debugging flags */
epicsExportAddress(int,TDS3000Debug);

#define TIMEWINDOW     2.00      /* 2 second delay before reconnect */
#define TIMEOUT        12.0      /* 12 second limit for I/O operation */

/******************************************************************************
 *
 * String arrays for EFAST operations.  Note that the last entry must be 
 * NULL.
 *
 * On input operations, only as many bytes as are found in the string array
 * elements are compared.  If there are more bytes than that in the input
 * message, they are ignored.  The first matching string found (starting
 * from the 0'th element) will be used as a match.
 *
 * NOTE: For the input operations, the strings are compared literally!  This
 * can cause problems if the instrument is returning things like \r and \n
 * characters.  You must take care when defining input strings so you include
 * them as well.
 *
 ******************************************************************************/

static char     *(timeRefCmd[])    = {"HOR:TRIG:POS 10",
                                      "HOR:TRIG:POS 50",
                                      "HOR:TRIG:POS 90",    
                                      NULL};

static char     *(trigModeCmd[])   = {"TRIG:A:MODE NORMAL",  
                                      "TRIG:A:MODE AUTO",
                                      NULL};

static char     *(trigModeTable[])   = {"NORMAL",
                                        "AUTO",
                                        NULL};

static char     *(trigSourceCmd[]) = {"TRIG:A:EDGE:SOU CH1",
                                      "TRIG:A:EDGE:SOU CH2",
                                      "TRIG:A:EDGE:SOU CH3",
                                      "TRIG:A:EDGE:SOU CH4",
                                      "TRIG:A:EDGE:SOU LINE",
                                      "TRIG:A:EDGE:SOU VERTICAL",
                                      "TRIG:A:EDGE:SOU EXT10",
                                      "TRIG:A:EDGE:SOU EXT",
                                      NULL};

static char     *(trigSourceTable[]) = {"CH1",
                                        "CH2",
                                        "CH3",
                                        "CH4",
                                        "LINE",
                                        "VERTICAL",
                                        "EXT10",
                                        "EXT",
                                        NULL};

static char     *(trigSlopeCmd[])  = {"TRIG:A:EDGE:SLO FALL",
                                      "TRIG:A:EDGE:SLO RISE",
                                      NULL};

static char     *(trigSlopeTable[])  = {"FALL",
                                        "RISE",
                                        NULL};

static char     *trigStat[] = {"AUTO",
                               "ARMED",
                               "READY",
                               "SAVE",
                               "TRIGGER",
                                NULL};

static char     *(chanVdivCmd[])  = {"SCALE 1.0E-3",
                                     "SCALE 2.0E-3",
                                     "SCALE 5.0E-3", 
                                     "SCALE 10.0E-3", 
                                     "SCALE 20.0E-3", 
                                     "SCALE 50.0E-3", 
                                     "SCALE 100.0E-3", 
                                     "SCALE 200.0E-3", 
                                     "SCALE 500.0E-3", 
                                     "SCALE 1", 
                                     "SCALE 2", 
                                     "SCALE 5", 
                                     "SCALE 10", 
                                      NULL};

static char     *(chanVdivTable[])   = {"1.0E-3",
                                        "2.0E-3",
                                        "5.0E-3", 
                                        "1.0E-2", 
                                        "2.0E-2", 
                                        "5.0E-2", 
                                        "1.0E-1", 
                                        "2.0E-1", 
                                        "5.0E-1", 
                                        "1.0E0", 
                                        "2.0E0", 
                                        "5.0E0", 
                                        "1.0E1", 
                                        NULL};


static char     *(chanCoupCmd[]) = {"COUPLING DC",
                                    "COUPLING AC", 
                                    "COUPLING GND",
                                     NULL};

static char     *(chanCoupleTable[])  = {"DC",
                                         "AC",
                                         "GND",
                                         NULL};


static char     *(chan1OnOffCmd[]) = {"SELECT:CH1 OFF",
                                      "SELECT:CH1 ON", 
                                      NULL};

static char     *(chan2OnOffCmd[]) = {"SELECT:CH2 OFF",
                                      "SELECT:CH2 ON", 
                                      NULL};

static char     *(chan3OnOffCmd[]) = {"SELECT:CH3 OFF",
                                      "SELECT:CH3 ON", 
                                      NULL};

static char     *(chan4OnOffCmd[]) = {"SELECT:CH4 OFF",
                                      "SELECT:CH4 ON", 
                                      NULL};

static char     *(chanImpedCmd[]) = {"IMP FIFTY",
                                     "IMP MEG", 
                                      NULL};

static char     *(chanImpedTable[]) = {"FIFTY",
                                       "MEG", 
                                       NULL};

static char     *(acquireStopAfterCmd[]) = {"ACQUIRE:STOPAFTER RUNSTOP",
                                            "ACQUIRE:STOPAFTER SEQUENCE", 
                                            NULL};

static char     *(acquireStopAfterTable[])  = {"RUNSTOP",
                                               "SEQUENCE",
                                               NULL};

static char     *(frontPanelLockCmd[])  = {"LOCK NONE",
                                           "LOCK ALL",
                                           NULL};

static char     *(frontPanelLockTable[])  = {"NONE",
                                             "ALL",
                                             NULL};

static char     *(recallCmd[])    = {"RECALL:SETUP 1", 
                                     "RECALL:SETUP 2", 
                                     "RECALL:SETUP 3", 
                                     "RECALL:SETUP 4", 
                                     "RECALL:SETUP 5",
                                     "RECALL:SETUP 6",
                                     "RECALL:SETUP 7",
                                     "RECALL:SETUP 8",
                                     "RECALL:SETUP 9",
                                     "RECALL:SETUP 10",
                                     "RECALL:SETUP FACTORY",
                                     NULL};

static char     *(saveCmd[])      = {"SAVE:SETUP 1",
                                     "SAVE:SETUP 2", 
                                     "SAVE:SETUP 3",
                                     "SAVE:SETUP 4",
                                     "SAVE:SETUP 5",
                                     "SAVE:SETUP 6",
                                     "SAVE:SETUP 7",
                                     "SAVE:SETUP 8",
                                     "SAVE:SETUP 9",
                                     "SAVE:SETUP 10",
                                     NULL};

static char     *printerPortTable[] = {"CENTRONICS",
                                       "FILE",
                                       "GIPB", 
                                       "RS232",
                                       "ETHERNET",
                                        NULL};

static char     *printerPortCmd[] = {"HARDC:PORT CENTRONICS",
                                     "HARDC:PORT FILE",
                                     "HARDC:PORT GIPB", 
                                     "HARDC:PORT RS232",
                                     "HARDC:PORT ETHERNET",
                                      NULL};

static char     *printerFormatTable[] = {"DESKJETC",
                                         "EPSCOLOR",
                                         "LASERJET", 
                                         "PCXCOLOR",
                                          NULL};

static char     *printerFormatCmd[] = {"HARDCOPY:FORMAT DESKJETC",
                                       "HARDCOPY:FORMAT EPSCOLOR",
                                       "HARDCOPY:FORMAT LASERJET", 
                                       "HARDCOPY:FORMAT PCXCOLOR",
                                        NULL};

static char     *(messageBoxCmd[])  = 
                        {"MESSAGE:SHOW 'RF1HV\n\nFront\nControls\nLOCKED'",
                         "MESSAGE:SHOW 'RF2HV\n\nFront\nControls\nLOCKED'",
                         "MESSAGE:SHOW 'RF3HV\n\nFront\nControls\nLOCKED'",
                         "MESSAGE:SHOW 'RF4HV\n\nFront\nControls\nLOCKED'",
                         "MESSAGE:SHOW 'RF1LL\n\nFront\nControls\nLOCKED'",
                         "MESSAGE:SHOW 'RF2LL\n\nFront\nControls\nLOCKED'",
                         "MESSAGE:SHOW 'RF3LL\n\nFront\nControls\nLOCKED'",
                         "MESSAGE:SHOW 'RF4LL\n\nFront\nControls\nLOCKED'",
                         NULL};

static char     *(messageBoxTable[])  = 
                        {"'RF1HV\nFront\nControls\nLOCKED'",
                         "'RF2HV\nFront\nControls\nLOCKED'",
                         "'RF3HV\nFront\nControls\nLOCKED'",
                         "'RF4HV\nFront\nControls\nLOCKED'",
                         "'RF1LL\nFront\nControls\nLOCKED'",
                         "'RF2LL\nFront\nControls\nLOCKED'",
                         "'RF3LL\nFront\nControls\nLOCKED'",
                         "'RF4LL\nFront\nControls\nLOCKED'",
                         NULL};


/****************************************************************************** *
 * Array of structures that define all GPIB messages
 * supported for this type of instrument.
 *
 ******************************************************************************/

/* forward declarations of some custom convert routines */
static int convertData();
static int scaleData();
static int preamblePoints();
static int preambleCount();
static int writeSetupData();
static int readSetupData();
static int writeStringWf();
static int getEthernetName();


static struct gpibCmd gpibCmds[] = 
{
/* Param */
/*  0 */ FILL,

/* Timebase */
/*  1 : read horizontal scale : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "HOR:MAIN:SCALE?", "%lf", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/*  2 : read delay : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "HOR:DEL:TIM?", "%lf", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/*  3 : set horizontal scale : AO */
         {&DSET_AO, GPIBWRITE, IB_Q_LOW, NULL, "HOR:MAIN:SCALE %e", 0, 30,
          NULL, 0, 0, NULL, NULL, 0},
/*  4 : set delay : AO */
         {&DSET_AO, GPIBWRITE, IB_Q_LOW, NULL,"HOR:DEL:TIM %e", 0, 30,
          NULL, 0, 0, NULL, NULL, 0},
/*  5 : set display reference : MBBO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, timeRefCmd, NULL, 0},
/*  6 : read display reference : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "HOR:TRIG:POS?", "%lf", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/*  7 - 9 :            */
         FILL,FILL,FILL,

/* Channel 1 */ 
/* 10 : read vertical range : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "CH1:SCALE?", "%lf", 0, 20,
          NULL, 0 ,0, NULL, NULL, NULL},
/* 11 : set vertical scale : MO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, "CH1:", NULL, 0, 30,
          NULL, 0, 0, chanVdivCmd, NULL, 0},
/* 12 : read offset range : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "CH1:OFFS?", "%lf", 0, 20,
          NULL, 0 ,0, NULL, NULL, NULL},
/* 13 : set offset : AO */
         {&DSET_AO, GPIBWRITE, IB_Q_LOW, NULL,"CH1:OFFS %e", 0, 30,
          NULL, 0, 0, NULL, NULL, 0},
/* 14 : set coulping : MO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, "CH1:", NULL, 0, 30,
          NULL, 0, 0, chanCoupCmd, NULL, 0},
/* 15 : enable channel : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, chan1OnOffCmd, NULL, 0},
/* 16 : read vertical range : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "CH1:SCALE?", NULL, 0, 50,
          NULL, 0, 0, chanVdivTable, NULL, NULL},
/* 17 : read coupling : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "CH1:COUPLING?", NULL, 0, 50,
          NULL, 0, 0, chanCoupleTable, NULL, NULL},
/* 18 : read channel enable : BI  */
         {&DSET_BI, GPIBREAD, IB_Q_HIGH, "SEL:CH1?", "%d", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 19 : read waveform : WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU CH1;:WAVFRM?", 
            "%lf", 0, 32000, convertData, 0 ,0, NULL, NULL, NULL},
/* 20 : read channel impedance : BI */
         {&DSET_BI, GPIBEFASTI, IB_Q_HIGH, "CH1:IMP?", NULL, 0, 50,
          NULL, 0, 0, chanImpedTable, NULL, NULL},
/* 21 : set channel impedance : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, "CH1:", NULL, 0, 30,
          NULL, 0, 0, chanImpedCmd, NULL, 0},
/* 22 - 29 :            */
         FILL,FILL,FILL,FILL,FILL,FILL,FILL,FILL,

/* Channel 2 */
/* 30 : read vertical range : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "CH2:SCALE?", "%lf", 0, 20,
          NULL, 0 ,0, NULL, NULL, NULL},
/* 31 : set vertical scale : MO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, "CH2:", NULL, 0, 30,
          NULL, 0, 0, chanVdivCmd, NULL, 0},
/* 32 : read offset range : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "CH2:OFFS?", "%lf", 0, 20,
          NULL, 0 ,0, NULL, NULL, NULL},
/* 33 : set offset : AO */
         {&DSET_AO, GPIBWRITE, IB_Q_LOW, NULL,"CH2:OFFS %e", 0, 30,
          NULL, 0, 0, NULL, NULL, 0},
/* 34 : set coulping : MO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, "CH2:", NULL, 0, 30,
          NULL, 0, 0, chanCoupCmd, NULL, 0},
/* 35 : enable channel : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, chan2OnOffCmd, NULL, 0},
/* 36 : read vertical range : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "CH2:SCALE?", NULL, 0, 50,
          NULL, 0, 0, chanVdivTable, NULL, NULL},
/* 37 : read coupling : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "CH2:COUPLING?", NULL, 0, 50,
          NULL, 0, 0, chanCoupleTable, NULL, NULL},
/* 38 : read channel enable : BI  */
         {&DSET_BI, GPIBREAD, IB_Q_HIGH, "SEL:CH2?", "%d", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 39 : read waveform : WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU CH2;:WAVFRM?", 
            "%lf", 0, 32000, convertData, 0 ,0, NULL, NULL, NULL},
/* 40 : read channel impedance : BI */
         {&DSET_BI, GPIBEFASTI, IB_Q_HIGH, "CH2:IMP?", NULL, 0, 50,
          NULL, 0, 0, chanImpedTable, NULL, NULL},
/* 41 : set channel impedance : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, "CH2:", NULL, 0, 30,
          NULL, 0, 0, chanImpedCmd, NULL, 0},
/* 42 - 49 :            */
         FILL,FILL,FILL,FILL,FILL,FILL,FILL,FILL,

/* Channel 3 */
/* 50 : read vertical range : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "CH3:SCALE?", "%lf", 0, 20,
          NULL, 0 ,0, NULL, NULL, NULL},
/* 51 : set vertical scale : MO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, "CH3:", NULL, 0, 30,
          NULL, 0, 0, chanVdivCmd, NULL, 0},
/* 52 : read offset range : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "CH3:OFFS?", "%lf", 0, 20,
          NULL, 0 ,0, NULL, NULL, NULL},
/* 53 : set offset : AO */
         {&DSET_AO, GPIBWRITE, IB_Q_LOW, NULL,"CH3:OFFS %e", 0, 30,
          NULL, 0, 0, NULL, NULL, 0},
/* 54 : set coulping : MO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, "CH3:", NULL, 0, 30,
          NULL, 0, 0, chanCoupCmd, NULL, 0},
/* 55 : enable channel : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, chan3OnOffCmd, NULL, 0},
/* 56 : read vertical range : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "CH3:SCALE?", NULL, 0, 50,
          NULL, 0, 0, chanVdivTable, NULL, NULL},
/* 57 : read coupling : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "CH3:COUPLING?", NULL, 0, 50,
          NULL, 0, 0, chanCoupleTable, NULL, NULL},
/* 58 : read channel enable : BI  */
         {&DSET_BI, GPIBREAD, IB_Q_HIGH, "SEL:CH3?", "%d", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 59 : read waveform : WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU CH3;:WAVFRM?", 
            "%lf", 0, 32000, convertData, 0 ,0, NULL, NULL, NULL},
/* 60 : read channel impedance : BI */
         {&DSET_BI, GPIBEFASTI, IB_Q_HIGH, "CH3:IMP?", NULL, 0, 50,
          NULL, 0, 0, chanImpedTable, NULL, NULL},
/* 61 : set channel impedance : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, "CH3:", NULL, 0, 30,
          NULL, 0, 0, chanImpedCmd, NULL, 0},
/* 62 - 69 :            */
         FILL,FILL,FILL,FILL,FILL,FILL,FILL,FILL,
 
/* Channel 4 */
/* 70 : read vertical range : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "CH4:SCALE?", "%lf", 0, 20,
          NULL, 0 ,0, NULL, NULL, NULL},
/* 71 : set vertical scale : MO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, "CH4:", NULL, 0, 30,
          NULL, 0, 0, chanVdivCmd, NULL, 0},
/* 72 : read offset range : AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "CH4:OFFS?", "%lf", 0, 20,
          NULL, 0 ,0, NULL, NULL, NULL},
/* 73 : set offset : AO */
         {&DSET_AO, GPIBWRITE, IB_Q_LOW, NULL,"CH4:OFFS %e", 0, 30,
          NULL, 0, 0, NULL, NULL, 0},
/* 74 : set coulping : MO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, "CH4:", NULL, 0, 30,
          NULL, 0, 0, chanCoupCmd, NULL, 0},
/* 75 : enable channel : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, chan4OnOffCmd, NULL, 0},
/* 76 : read vertical range : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "CH4:SCALE?", NULL, 0, 50,
          NULL, 0, 0, chanVdivTable, NULL, NULL},
/* 77 : read coupling : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "CH4:COUPLING?", NULL, 0, 50,
          NULL, 0, 0, chanCoupleTable, NULL, NULL},
/* 78 : read channel enable : BI  */
         {&DSET_BI, GPIBREAD, IB_Q_HIGH, "SEL:CH4?", "%d", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 79 : read waveform : WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU CH4;:WAVFRM?", 
            "%lf", 0, 32000, convertData, 0 ,0, NULL, NULL, NULL},
/* 80 : read channel impedance : BI */
         {&DSET_BI, GPIBEFASTI, IB_Q_HIGH, "CH4:IMP?", NULL, 0, 50,
          NULL, 0, 0, chanImpedTable, NULL, NULL},
/* 81 : set channel impedance : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, "CH4:", NULL, 0, 30,
          NULL, 0, 0, chanImpedCmd, NULL, 0},
/* 82 - 89 :            */
         FILL,FILL,FILL,FILL,FILL,FILL,FILL,FILL,

/* Instrument Parameters: Part 1  */
/* 90 : init GPIB Parameters */
         {&DSET_BO, GPIBCMD, IB_Q_HIGH, 
         "*CLS;:WFMP:ENC BIN; BN_F RI; BYT_N 2; BYT_O MSB;:HOR:RECORD 10000;:HEAD OFF;:VERBOSE ON;:DATA:START 1;:DATA:STOP 10000",
          NULL, 0, 0, NULL, 0 ,0, NULL, NULL, 0},
/* 91 : recall memory setting : MBBO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, recallCmd, NULL, 0},
/* 92 : save memory setting : MBBO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, saveCmd, NULL, 0},
/* 93 : string output only : SO */
         {&DSET_SO, GPIBWRITE, IB_Q_HIGH, NULL, "%s", 0, 41,
          NULL, 0 ,0, NULL, NULL, 0},
/* 94 : autoscale : BO */
         {&DSET_BO, GPIBCMD, IB_Q_HIGH, "AUTOSET", NULL, 0, 0,
          NULL, 0 ,0, NULL, NULL, 0},
/* 95 - 96 */
         FILL,FILL,
/* 97 : read setup : WF */
         {&DSET_WF, GPIBREAD, IB_Q_LOW, "SET?", "%lf", 0, 2060,
          readSetupData, 0 ,0, NULL, NULL, NULL},
/* 98 : write setup : WF */
         {&DSET_WF, GPIBWRITE, IB_Q_LOW, NULL, NULL, 0, 2068,
          writeSetupData, 0 ,0, NULL, NULL, 0},
/* 99 : reset scope : BO  */
         {&DSET_BO, GPIBCMD, IB_Q_HIGH, "*RST",
          NULL, 0, 0, NULL, 0 ,0, NULL, NULL, 0},

/* Trigger */
/* 100 : set time mode : MBBO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, trigModeCmd, NULL, 0},
/* 101 : set trigger source : MBBO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, trigSourceCmd, NULL, 0},
/* 102 : set trigger level : AO */
         {&DSET_AO, GPIBWRITE, IB_Q_LOW, NULL,"TRIG:A:LEV %e", 0, 30,
          NULL, 0, 0, NULL, NULL, 0},
/* 103 : set trigger slope : MBBO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, trigSlopeCmd, NULL, 0},
/* 104 : set holdoff : AO */
         {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL,"TRIG:A:HOLD:TIME %e", 0, 50,
          NULL, 0, 0, NULL, NULL, 0},
/* 105 : read holdoff : AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "TRIG:A:HOLD:TIME?","%lf", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 106 : read trigger level : AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "TRIG:A:LEV?","%lf", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 107 : read time mode : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "TRIG:A:MODE?", NULL, 0, 50,
          NULL, 0, 0, trigModeTable, NULL, NULL},
/* 108 : read trigger source mode : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "TRIG:A:EDGE:SOU?", NULL, 0, 50,
          NULL, 0, 0, trigSourceTable, NULL, NULL},
/* 109 : read trigger slope : MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "TRIG:A:EDGE:SLO?", NULL, 0, 50,
          NULL, 0, 0, trigSlopeTable, NULL, NULL},
/* 110 : Trigger Status */
         {&DSET_MBBI, GPIBEFASTI,  IB_Q_HIGH, "TRIGGER:STATE?", NULL, 0, 50,
          NULL, 0, 0, trigStat, NULL, NULL},
/* 111 - 119 */
         FILL,FILL,FILL,FILL,FILL,FILL,FILL,FILL,FILL,

/* Instrument Parameters Continued */
/* 120 : read GPIB device response : SI  */
         {&DSET_SI, GPIBREAD, IB_Q_HIGH, "", "%s", 0, 100,
          NULL, 0, 0, NULL, NULL, NULL},
/* 121 : XINCrement : AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "WFMPRE:XINCR?","%lf", 0, 100,
          NULL, 0, 0, NULL, NULL, NULL},
/* 122 : XORigin : AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "WFMPRE:PT_OFF?","%lf", 0, 100,
          NULL, 0, 0, NULL, NULL, NULL},
/* 123 : XREFerence : AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "WFMPRE:XZERO?","%lf", 0, 100,
          NULL, 0, 0, NULL, NULL, NULL},
/* 124 : YINCrement : AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "WFMPRE:YMULT?","%lf", 0, 100,
          NULL, 0, 0, NULL, NULL, NULL},
/* 125 : YORigin : AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "WFMPRE:YZERO?","%lf", 0, 100,
          NULL, 0, 0, NULL, NULL, NULL},
/* 126 : YREFerence : AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "WFMPRE:YOFF?","%lf", 0, 100,
          NULL, 0, 0, NULL, NULL, NULL},
/* 127 - 129 */
         FILL,FILL,FILL,

/* Waveform memory */
/* 130 : read waveform mem 1: WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU REF1;:WAVFRM?", 
            "%lf", 0, 32000, convertData, 0 ,0, NULL, NULL, NULL},
/* 131 : read waveform mem 2: WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU REF2;:WAVFRM?", 
            "%lf", 0, 32000, convertData, 0 ,0, NULL, NULL, NULL},
/* 132 : read waveform mem 3: WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU REF3;:WAVFRM?", 
            "%lf", 0, 32000, convertData, 0 ,0, NULL, NULL, NULL},
/* 133 : read waveform mem 4: WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU REF4;:WAVFRM?", 
            "%lf", 0, 32000, convertData, 0 ,0, NULL, NULL, NULL},
/* 134 : NPOInts (num points per segment from preamble): AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "WFMPRE?","%lf", 0, 300,
          preamblePoints, 0, 0, NULL, NULL, NULL},
/* 135 : NCOUnt (number of segments from preamble): AI */
         {&DSET_AI, GPIBREAD, IB_Q_HIGH, "WFMPRE?","%lf", 0, 300,
          preambleCount, 0, 0, NULL, NULL, NULL},

/* Waveforms for medm display */
/* 136 : read waveform : WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU CH1;:WAVFRM?", 
            "%lf", 0, 32000, scaleData, 0 ,0, NULL, NULL, NULL},
/* 137 : read waveform : WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU CH2;:WAVFRM?", 
            "%lf", 0, 32000, scaleData, 0 ,0, NULL, NULL, NULL},
/* 138 : read waveform : WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU CH3;:WAVFRM?", 
            "%lf", 0, 32000, scaleData, 0 ,0, NULL, NULL, NULL},
/* 139 : read waveform : WF */
        {&DSET_WF, GPIBCVTIO, IB_Q_LOW, "DAT:SOU CH4;:WAVFRM?", 
            "%lf", 0, 32000, scaleData, 0 ,0, NULL, NULL, NULL},
/* 140 : X axis in time units  */
        {&DSET_WF, GPIBREAD, IB_Q_LOW, "DAT:SOU CH1;:WFMPRE?", 
            "%lf", 0, 32000, scaleData, 1 ,0, NULL, NULL, NULL},
/* 141 - 149:            */
         FILL,FILL,FILL,FILL,FILL,FILL,FILL,FILL,FILL,

/* waveform records to increase the size of "stringin/stringout" capabilities */
/* 150 : read a long response string */
         {&DSET_WF, GPIBREAD, IB_Q_HIGH, "", NULL, 0, 258,
          readSetupData, 256, 0, NULL, NULL, NULL},
/* 151 : write a long response string */
         {&DSET_WF, GPIBWRITE, IB_Q_HIGH, NULL, NULL, 0, 258,
          writeStringWf, 256, 0, NULL, NULL, 0},

/* printer functions for a DeskJet color printer */
/* 152 - setup Hard copy to output to printer BO */
        {&DSET_BO, GPIBCMD,  IB_Q_HIGH, 
        "HARDC:INKS ON;LAY LAN;PALE NORM", 
        NULL, 0, 0, NULL, 0, 0, NULL, NULL , 0},
/* 153 - print to printer BO */
        {&DSET_BO, GPIBCMD, IB_Q_HIGH, ":CLEARMENU;:DIS:CLOCK ON;:HARDC START", 
        NULL, 0, 0, NULL, 0, 0, NULL, NULL , 0},
/* 154 - read printer port configuration MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_LOW, "HARDCOPY:PORT?", NULL, 0, 50,
          NULL, 0, 0, printerPortTable, NULL, NULL},
/* 155 - abort print job BO */
        {&DSET_BO, GPIBCMD, IB_Q_HIGH, "HARDCOPY ABORT", 
        NULL, 0, 0, NULL, 0, 0, NULL, NULL , 0},
/* 156 - clear menu on display BO */
        {&DSET_BO, GPIBCMD, IB_Q_HIGH, ":CLEARMENU", 
        NULL, 0, 0, NULL, 0, 0, NULL, NULL , 0},
/* 157 - set printer port : MBBO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, printerPortCmd, NULL, 0},
/* 158 - set printer format : MBBO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, printerFormatCmd, NULL, 0},
/* 159 - read printer format MBBI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_LOW, "HARDCOPY:FORMAT?", NULL, 0, 50,
          NULL, 0, 0, printerFormatTable, NULL, NULL},

/* Acquire Functions */
/* 160 : set acquire stop after : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, acquireStopAfterCmd, NULL, 0},
/* 161 : read acquire stop after status BI */
         {&DSET_BI, GPIBEFASTI, IB_Q_HIGH, "ACQUIRE:STOPAFTER?", NULL, 0, 50,
          NULL, 0, 0, acquireStopAfterTable, NULL, NULL},
/* 162 : start acquisition : BO */
         {&DSET_BO, GPIBWRITE, IB_Q_HIGH, NULL, "ACQUIRE:STATE RUN", 0, 20,
          NULL, 0 ,0, NULL, NULL, 0},
/* 163 : stop acquisition : BO */
         {&DSET_BO, GPIBWRITE, IB_Q_HIGH, NULL, "ACQUIRE:STATE STOP", 0, 20,
          NULL, 0 ,0, NULL, NULL, 0},
/* 164 : read acquire status BI */
         {&DSET_BI, GPIBREAD, IB_Q_HIGH, "ACQ:STATE?", "%d", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},

/* 165 - 169:            */
         FILL,FILL,FILL,FILL,FILL,

/* Miscellaneous Functions */
/* 170 : Front panel lock control : BO */
         {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, frontPanelLockCmd, NULL, 0},
/* 171 : Front panel lock status BI */
         {&DSET_BI, GPIBEFASTI, IB_Q_HIGH, "LOCK?", NULL, 0, 50,
          NULL, 0, 0, frontPanelLockTable, NULL, NULL},
/* 172 : read BUSY status, for printing or acquire status BI */
         {&DSET_BI, GPIBREAD, IB_Q_HIGH, "BUSY?", "%d", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 173 : read content of message box SI */
         {&DSET_SI, GPIBREAD, IB_Q_HIGH, "MESSAGE:SHOW?", "%s", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 174 : write content to message box SO */
         {&DSET_SO, GPIBWRITE, IB_Q_HIGH, NULL, "MESSAGE:SHOW %s", 0, 20,
          NULL, 0, 0, NULL, NULL, 0},
/* 175 : config message box BO */
         {&DSET_BO, GPIBWRITE, IB_Q_HIGH, NULL, "MESSAGE:STATE %d", 0, 20,
          NULL, 0, 0, NULL, NULL, 0},
/* 176 : message box status BI */
         {&DSET_BI, GPIBREAD, IB_Q_HIGH, "MESSAGE:STATE?", "%d", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 177 : read message box MI */
         {&DSET_MBBI, GPIBEFASTI, IB_Q_HIGH, "MESSAGE:SHOW?", NULL, 0, 50,
          NULL, 0, 0, messageBoxTable, NULL, NULL},
/* 178 : write message box : MO */
         {&DSET_MBBO, GPIBEFASTO, IB_Q_HIGH, NULL, NULL, 0, 0,
          NULL, 0, 0, messageBoxCmd, NULL, 0},
/* 179 : config message box size BO */
         {&DSET_BO, GPIBWRITE, IB_Q_HIGH, NULL, 
	 "MESSAGE:BOX 540,300,630,400", 0, 50, NULL, 0, 0, NULL, NULL, 0},
/* 180 : read ethernet name SI */
         {&DSET_SI, GPIBCVTIO, IB_Q_HIGH, NULL, NULL, 0, 100, 
          getEthernetName, 0, 0, NULL, NULL, NULL},
/* 181 : write SO */
         {&DSET_SO, GPIBWRITE, IB_Q_HIGH, NULL, "%s", 0, 100,
          NULL, 0, 0, NULL, NULL, 0},
/* 182 : read Device Event Status Enable Register: AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "DESE?", "%lf", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 183 : read Standard Event Status Register: AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "*ESR?", "%lf", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 184 : read Event Queue: AI */
         {&DSET_AI, GPIBREAD, IB_Q_LOW, "EVENT?", "%lf", 0, 20,
          NULL, 0, 0, NULL, NULL, NULL},
/* 185 : set Device Event Status Enable Register : AO */
         {&DSET_AO, GPIBWRITE, IB_Q_LOW, NULL, "DESE %f", 0, 30,
          NULL, 0, 0, NULL, NULL, 0},
/* 186 : clear Event Status : BO  */
         {&DSET_BO, GPIBCMD, IB_Q_HIGH, "*CLS",
          NULL, 0, 0, NULL, 0 ,0, NULL, NULL, 0}

};


/* The following is the number of elements in the command array above.  */
#define NUMPARAMS       sizeof(gpibCmds)/sizeof(struct gpibCmd)


/*****************************************************************************
 *
 * Structure containing the user's functions and operating parameters needed
 * by the gpib library functions.
 *
 *****************************************************************************/
static long init_ai(int parm)
{
   if (parm == 0) {
     devSupParms.name = "devTDS3000Gpib"; /* device support module type name */
     devSupParms.gpibCmds = gpibCmds;     /* GPIB command array */
     devSupParms.numparams = NUMPARAMS;   /* number of supported parameters */
     devSupParms.timeout = TIMEOUT;       /* I/O time limit (msec) */
     devSupParms.timeWindow = TIMEWINDOW; /* # of milliseconds to skip after a device times out */
     devSupParms.respond2Writes = -1;     /* set pos if the device responds to writes */
   }
   return(0);
}


/****************************************************************************
 *
 * 
 * Custom convert routines for TDS3000 
 *
 *
 ***************************************************************************/

/* NOT USED
static int gpibResponse(pdpvt, P1, P2, P3)
  struct gpibDpvt *pdpvt;
  int P1;
  int P2;
  char **P3;
{
  printf("gpib response : %s \n",pdpvt->rsp);
  return(2);
}
*/


/* Use waveform preamble data to interpret waveform */
static int convertData(pdpvt, P1, P2, P3)
  struct gpibDpvt *pdpvt;
  int P1;
  int P2;
  char **P3;
{ 
  double        xIncr, xZero, yMult, yZero, yOff;
  int byte, bits, length, ptOff, width;
  char encode[20], binFmt[20], byteOr[20], idString[80], ptFmt[20], xUnit[20], yUnit[20];
  int len, i;
  size_t ntrans;
  struct waveformRecord *pwf= (struct waveformRecord *) (pdpvt->precord);
    gpibCmd *pgpibCmd = gpibCmdGet(pdpvt);
    asynOctet *pasynOctet = pdpvt->pasynOctet;
    void *asynOctetPvt = pdpvt->asynOctetPvt;

    if ((pasynOctet->write(asynOctetPvt,pdpvt->pasynUser,pgpibCmd->cmd,
                                      strlen(pgpibCmd->cmd),&ntrans) != asynSuccess)
     || (pdevSupportGpib->readArbitraryBlockProgramData(pdpvt) <= 0)) {
        recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
        return -1;
    }

  if (sscanf(pdpvt->msg, "%d;%d;%19[^;];%19[^;];%19[^;];%d;%79[^;];%19[^;];"
                         "%lg;%d;%lg;%19[^;];%lg;%lg;%lg;%19[^;];#%1d%n",
         &byte, &bits, encode, binFmt, byteOr, &length, idString, ptFmt,
         &xIncr, &ptOff, &xZero, xUnit, &yMult, &yZero, &yOff, yUnit, &width, &i
            ) != 17) {
    recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
    return -1;
  }
  if(TDS3000Debug) {
      printf("%p, index=%d\n", (unsigned char *)&pdpvt->msg[0],i);
      printf("byte- %d, bits- %d, encode- %s, binFmt- %s, byteOr-  %s\n",
              byte, bits, encode, binFmt, byteOr);
      printf("length- %d, ptFmt- %s\n",
              length, ptFmt);
      printf("xIncr- %g\nptOff- %d\nxZero- %g\nyMult- %g\nyZero- %g\nyOff- %g\n"              , xIncr, ptOff, xZero, yMult, yZero, yOff);
  }
  i += width; /* I should be pointing to the first data byte now */

  /* Copy waveform data from response buffer to waveform record */
  if(P1) {
      printf("%p, index=%d\n", (void *)&pdpvt->msg[0], i);
      printf("ptFmt - %s, length - %d\n", ptFmt, length);
      len = P1;   /* if P1 is non-zero, use it as length */
  }
  else {
      len = length;
  }
  memcpy((char *)pwf->bptr, (char *)&pdpvt->msg[i], len*byte);
  pwf->nord=len;
  return(0);
}


/* Use waveform preamble to scale waveforms to volts waveform */
/* Also, read only the first .NELM values */
static int scaleData(pdpvt, P1, P2, P3)
  struct gpibDpvt *pdpvt;
  int P1;
  int P2;
  char **P3;
{
  signed char    *pByte;
  signed short   *pShort;
  float          *pFArray, *pFTime;
  double        xIncr, xZero, yMult, yZero, yOff, temp;
  int byte, bits, length, ptOff, width;
  char encode[20], binFmt[20], byteOr[20], idString[80], ptFmt[20], xUnit[20], yUnit[20];
  int i, j;
  struct waveformRecord *pwf= (struct waveformRecord *) (pdpvt->precord);
    gpibCmd *pgpibCmd = gpibCmdGet(pdpvt);
    asynOctet *pasynOctet = pdpvt->pasynOctet;
    void *asynOctetPvt = pdpvt->asynOctetPvt;
  size_t ntrans;

    if (P1 == 0) {
        if ((pasynOctet->write(asynOctetPvt,pdpvt->pasynUser,pgpibCmd->cmd,
                                    strlen(pgpibCmd->cmd),&ntrans) != asynSuccess)
         || (pdevSupportGpib->readArbitraryBlockProgramData(pdpvt) <= 0)) {
            recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
            return -1;
        }
    }

  if ((j = sscanf(pdpvt->msg,"%d;%d;%19[^;];%19[^;];%19[^;];%d;%79[^;];%19[^;];"
                         "%lg;%d;%lg;%19[^;];%lg;%lg;%lg;%19[^;];%n",
         &byte, &bits, encode, binFmt, byteOr, &length, idString, ptFmt,
         &xIncr, &ptOff, &xZero, xUnit, &yMult, &yZero, &yOff, yUnit, &i
            )) != 16) {
    printf ("Waveform preamble scanned %d of 16 (\"%.*s\").\n", j, pdpvt->msgInputLen,pdpvt->msg);
    recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
    return -1;
  }
  if(TDS3000Debug) {
     printf("%s\n", pwf->name);
     printf("%p, index=%d\n", (unsigned char *)&pdpvt->msg[0],i);
     printf("byte- %d, bits- %d, encode- %s, binFmt- %s, byteOr-  %s\n",
		     byte, bits, encode, binFmt, byteOr);
     printf("length- %d, ptFmt- %s\n",
             length, ptFmt);
     printf("xIncr- %g\nptOff- %d\nxZero- %g\nyMult- %g\nyZero- %g\nyOff- %g\n",                    xIncr, ptOff, xZero, yMult, yZero, yOff);
  }

  switch(P1) {
    case 0: /* Channel Data */
      /* Confirm wf record is expecting floats */
      if(pwf->ftvl != menuFtypeFLOAT) {
          printf("%s.FTVL must be FLOAT\n",pwf->name);
          recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
          return -1;
      }
      if (sscanf(pdpvt->msg+i, "#%1d", &width) != 1) {
        printf ("Failed to scan waveform width (\"%.*s\"[%d].\n", 
			    pdpvt->msgInputLen, pdpvt->msg, i);
        recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
        return -1;
      }
      i += 2 + width; /* I should be pointing to the first data byte now */
      if(TDS3000Debug)
        printf("width- %d    i- %d\n", width, i);
      /* initialize byte pointer and short pointer */
      pByte  = (signed char *)&pdpvt->msg[i];
      pShort= (short *)&pdpvt->msg[i];
      pFArray = (float *)pwf->bptr;
      /* Fill in waveform */
      switch(byte) {
        case 1: /* data type = BYTE */
          for(j=0; ((j<length) && (j<pwf->nelm)); j++, pByte++ ) {
              /* The following conversion yields VOLTS */
              pFArray[j] = (float)(((*pByte - yOff) * yMult) + yZero); 
              /* The following conversion yields DIVISIONS 
              pFArray[j] = (float)(((*pByte - yOff)/yOff * 5.0)); */
          }
          pwf->nord=j;
          break;
        case 2: /* data type = WORD  */
          for(j=0; ((j<length) && (j<pwf->nelm)); j++, pShort++) {
              unsigned char *c1 = (unsigned char *)pShort; 
              unsigned char *c2 = c1 + 1;
              short stmp = (*c1 << 8) | *c2; /* WFMPre:BYT_OR==MSB */
              /* The following conversion yields VOLTS */
              pFArray[j] = (float)(((stmp - yOff) * yMult) + yZero);
              /* The following conversion yields DIVISIONS 
              pFArray[j] = (float)(((stmp - yOff)/yOff * 5.0)); */
          }
          pwf->nord=j;
          break;

        default : /* invalid size */
          printf("%s bad WFRMPre:BYT_Nr.\n",pwf->name);
          recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
          return -1;
      }
      break;
      
    case 1: /* Time axis data, in nanoseconds */
      /* Confirm wf record is expecting doubles */
      /*if(pwf->ftvl != menuFtypeDOUBLE) {
          printf("%s.FTVL must be DOUBLE\n",pwf->name);
          recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
          return -1;
      }*/
      if(pwf->ftvl != menuFtypeFLOAT) {
          printf("%s.FTVL must be FLOAT\n",pwf->name);
          recGblSetSevr(pwf, READ_ALARM, INVALID_ALARM);
          return -1;
      }
      pFTime = (float *)pwf->bptr; 
      for(j=0; ((j<length) && (j<pwf->nelm)); j++) {
          /* The following conversion yields TIME in nanoseconds */
          /* E. NORUM -- Why not leave the units as seconds??? */
          /* temp = (float)((((j-ptOff) * xIncr) + xZero) * 1.0e9); */
          temp = (float)((((j-ptOff) * xIncr) + xZero) * 1.0e6); 
          pFTime[j] = temp;
          if(TDS3000Debug) {
              printf("%d,%6.0f ", j, pFTime[j]);
          }
          /* The following conversion yields DIVISIONS   
          pFTime[j] = (float)(j*(10.0/length));  */
      }
      if(TDS3000Debug) {
          printf("\nTotal count = %d\n ", j);
      }
      pwf->nord=j;
      break;
  }
  return(0);
}


static int preamblePoints(pdpvt, P1, P2, P3)
  struct gpibDpvt *pdpvt;
  int P1;
  int P2;
  char **P3;
{ 
  char preamble[255];
  char *token;
  int byte, bits, length;
  char *binFmt, *byteOr;
  long i;
  struct aiRecord *pai= (struct aiRecord *) (pdpvt->precord);

  /* Get waveform preamble  */
  i = 0;
  while (pdpvt->msg[i] != '\0') {
    preamble[i] = pdpvt->msg[i];
    i++;
    if (i == 254) break;
  }
  preamble[i] = '\0';
  token = strtok(&(preamble[0]),";");
  byte = atoi(token);           /* 1 or 2 byte per point */
  token = strtok(NULL,";");
  bits = atoi(token);           /* 8 or 16 bits per point */
  token = strtok(NULL,";");
  binFmt = token;               /* ASCII or Binary */
  token = strtok(NULL,";");
  byteOr = token;               /* byte order */
  token = strtok(NULL,";");
  length = atoi(token);         /* 500 or 10k points per WF */

  /* Copy length (points) value to val field of ai record */
  pai->val = (double)length;

  return(0);
}


static int preambleCount(pdpvt, P1, P2, P3)
  struct gpibDpvt *pdpvt;
  int P1;
  int P2;
  char **P3;
{ 
  int count;
  struct aiRecord *pai= (struct aiRecord *) (pdpvt->precord);

  /* This Tek scope doesn't have a count value */
  count = 1;

  /* Copy count value to val field of ai record */
  pai->val = (double)count;

  return(0);
}

static int readSetupData(pdpvt, P1, P2, P3)
  struct gpibDpvt *pdpvt;
  int P1;
  int P2;
  char **P3;
{ 
  struct waveformRecord *pwf= (struct waveformRecord *) (pdpvt->precord);
  int ncopy;

  if(pdpvt->msgInputLen <= pwf->nelm) {
    ncopy = pdpvt->msgInputLen;
  }
  else {
    ncopy = pwf->nelm;
    errlogPrintf("%s readSetupData() truncating %d bytes to %lu byte .NELM\n",
                                    pwf->name, pdpvt->msgInputLen, pwf->nelm);
  }
  memcpy(pwf->bptr,&pdpvt->msg[0],ncopy);
  pwf->nord=ncopy;
  return(0);
}

static int writeSetupData(pdpvt, P1, P2, P3)
  struct gpibDpvt *pdpvt;
  int P1;
  int P2;
  char **P3;
{ 
  struct waveformRecord *pwf= (struct waveformRecord *) (pdpvt->precord);

  strncpy(&pdpvt->msg[0],":SYST:SET ",10);
  memcpy(&pdpvt->msg[10],pwf->bptr,pwf->nord);
  return(pwf->nord+10);
}

static int writeStringWf(pdpvt, P1, P2, P3)
  struct gpibDpvt *pdpvt;
  int P1;
  int P2;
  char **P3;
{ 
  struct waveformRecord *pwf= (struct waveformRecord *) (pdpvt->precord);
  int nelm;

  nelm = (pwf->nord < P1)?pwf->nord:P1;
  memcpy(&pdpvt->msg[0],pwf->bptr,nelm);
  return(nelm);
}

static int
getEthernetName(struct gpibDpvt *pdpvt, int P1, int P2, char **P3)
{
    struct stringinRecord *psi= (struct stringinRecord *) (pdpvt->precord);

    asynOctet *pasynOctet = pdpvt->pasynOctet;
    void *asynOctetPvt = pdpvt->asynOctetPvt;
    int j;
    size_t ntrans;
    char cbuf[300];
    char make[12], model[12];
    char netName[40];

    if ((pasynOctet->write(asynOctetPvt,pdpvt->pasynUser,"*IDN?",
                                sizeof"*IDN?" - 1,&ntrans) != asynSuccess)
     || (pasynOctet->read(asynOctetPvt,pdpvt->pasynUser,cbuf,
                              sizeof cbuf - 1,&ntrans,NULL) != asynSuccess)) {
        recGblSetSevr(psi, READ_ALARM, INVALID_ALARM);
        return -1;
    }
    cbuf[ntrans] = '\0';   /* Null-terminate reply message */
    /* do the checks in cbuf and optionally send the "ETHERNET:NAME?"
       using the write/read commands like above */

    if (( j = sscanf(cbuf, "%11[^,],%11[^,]", make, model)) != 2){
      printf("Scope ERROR: Scope ID scanned %d, min 2 needed. ID string = \n %s \n"
		 , j, cbuf);
      return -1;
    }

    if(TDS3000Debug) {
      printf("cbuf = %s\n",cbuf);
      printf("make = %s\n",make);
      printf("model = %s\n",model);
      printf("model[8] = %c\n",model[8]);
    }

    if ( model[8] == 'B' || strstr(cbuf,"TDS3EM") != NULL ){ /* has ethernet option */
      if ((pasynOctet->write(asynOctetPvt,pdpvt->pasynUser,"ETHER:NAME?",
                                sizeof"ETHER:NAME?" - 1,&ntrans) != asynSuccess)
           || (pasynOctet->read(asynOctetPvt,pdpvt->pasynUser,cbuf,
                                  sizeof cbuf - 1,&ntrans,NULL) != asynSuccess)) 
      {
        recGblSetSevr(psi, READ_ALARM, INVALID_ALARM);
        return -1;
      }
      cbuf[ntrans] = '\0';   /* Null-terminate reply message */
      sscanf(cbuf, "%39[^\n\r]",netName);
    }
    else {
      sscanf(cbuf, "%18[^\n\r]",netName);
    }
    strcpy(psi->val,netName);

    return 0;
} 
